@@ -912,14 +912,24 @@ class BwaIndexer(object):
     BWTALGO_BWTSW = 2
     BWTALGO_IS = 3
 
-    def __init__(self, algorithm="auto", block_size=10000000):
+    def __init__(
+        self,
+        algorithm="auto",
+        block_size=10000000,
+        capture_progress=True,
+        verbose=1,
+    ):
         """Initialize BWA indexer.
 
-        :param algorithm: BWT construction algorithm ('auto', 'rb2', 'bwtsw', 'is')
-        :param block_size: Block size for bwtsw algorithm (in bytes)
+        :param algorithm: BWT construction algorithm ('auto', 'rb2', 'bwtsw', 'is'; default: 'auto')
+        :param block_size: Block size for bwtsw algorithm in bytes (default: 10000000)
+        :param capture_progress: Capture progress messages during indexing (default: True)
+        :param verbose: BWA verbosity level - 0=silent, 1=quiet, 2=normal, 3+=debug (default: 1)
         """
         self.algorithm = algorithm
         self.block_size = block_size
+        self.capture_progress = capture_progress
+        self.verbose = verbose
 
         # Convert algorithm string to integer
         algo_map = {
@@ -945,18 +955,16 @@ class BwaIndexer(object):
             "messages": [],
         }
 
-    def build_index(self, fasta_file, prefix=None, capture_progress=True):
+    def build_index(self, fasta_file, prefix=None):
         """Build BWA index from FASTA file.
 
         :param fasta_file: Path to input FASTA file
         :param prefix: Output prefix for index files (default: same as FASTA file)
-        :param capture_progress: Capture progress messages (default: True)
         :returns: Path to the index prefix
         """
         import os
         import sys
-        import re
-        from io import StringIO
+        import threading
 
         if not os.path.exists(fasta_file):
             raise FileNotFoundError(f"FASTA file not found: {fasta_file}")
@@ -978,42 +986,42 @@ class BwaIndexer(object):
         fasta_bytes = fasta_file.encode("utf-8")
         prefix_bytes = prefix.encode("utf-8")
 
-        if capture_progress:
-            # Capture stderr to parse progress messages
-            import subprocess
-            import threading
-            
-            # Create a pipe to capture stderr
+        # Set BWA verbosity level
+        libbwa.bwa_verbose = int(self.verbose)
+
+        if self.capture_progress:
+            # Use os.pipe() to capture stderr from C code
+            # Threading is necessary because:
+            # 1. C code writes to stderr (file descriptor 2)
+            # 2. Pipe buffers are limited (~64KB)
+            # 3. If we don't read continuously, the pipe fills and C code blocks
+            # 4. We can't read after C completes because data is streamed
             read_fd, write_fd = os.pipe()
-            old_stderr = os.dup(2)  # Save original stderr
-            os.dup2(write_fd, 2)  # Redirect stderr to pipe
+            old_stderr = os.dup(2)
+            os.dup2(write_fd, 2)
             os.close(write_fd)
             
-            # Thread to read from pipe
-            captured_lines = []
-            
             def reader():
+                """Read stderr in separate thread to prevent pipe deadlock."""
                 with os.fdopen(read_fd, 'r', errors='replace') as f:
                     for line in f:
-                        captured_lines.append(line.rstrip())
-                        self._parse_progress_line(line)
+                        line = line.rstrip()
+                        if line:
+                            self._parse_progress_line(line)
             
             reader_thread = threading.Thread(target=reader, daemon=True)
             reader_thread.start()
             
             try:
-                # Call the C function
                 result = libbwa.bwa_idx_build(
                     fasta_bytes, prefix_bytes, self.algo_type, self.block_size
                 )
             finally:
-                # Restore stderr
                 sys.stderr.flush()
                 os.dup2(old_stderr, 2)
                 os.close(old_stderr)
                 reader_thread.join(timeout=1)
         else:
-            # Call without capturing
             result = libbwa.bwa_idx_build(
                 fasta_bytes, prefix_bytes, self.algo_type, self.block_size
             )
@@ -1068,55 +1076,6 @@ class BwaIndexer(object):
             return (self.progress["characters_processed"] / self.progress["text_length"]) * 100
         return None
 
-    def build_index_with_options(
-        self, fasta_file, prefix=None, algorithm=None, block_size=None
-    ):
-        """Build BWA index with specific options.
-
-        :param fasta_file: Path to input FASTA file
-        :param prefix: Output prefix for index files
-        :param algorithm: BWT construction algorithm ('auto', 'rb2', 'bwtsw', 'is')
-        :param block_size: Block size for bwtsw algorithm
-        :returns: Path to the index prefix
-        """
-        # Temporarily override instance settings
-        old_algorithm = self.algorithm
-        old_block_size = self.block_size
-
-        if algorithm is not None:
-            self.algorithm = algorithm
-            algo_map = {
-                "auto": self.BWTALGO_AUTO,
-                "rb2": self.BWTALGO_RB2,
-                "bwtsw": self.BWTALGO_BWTSW,
-                "is": self.BWTALGO_IS,
-            }
-            if algorithm not in algo_map:
-                raise ValueError(
-                    f"Unknown algorithm '{algorithm}'. "
-                    f"Choose from: {list(algo_map.keys())}"
-                )
-            self.algo_type = algo_map[algorithm]
-
-        if block_size is not None:
-            self.block_size = block_size
-
-        try:
-            result = self.build_index(fasta_file, prefix)
-        finally:
-            # Restore original settings
-            self.algorithm = old_algorithm
-            self.block_size = old_block_size
-            if algorithm is not None:
-                algo_map = {
-                    "auto": self.BWTALGO_AUTO,
-                    "rb2": self.BWTALGO_RB2,
-                    "bwtsw": self.BWTALGO_BWTSW,
-                    "is": self.BWTALGO_IS,
-                }
-                self.algo_type = algo_map[self.algorithm]
-
-        return result
 
 
 def get_parser():
